@@ -1,7 +1,7 @@
 /*
  * config - configuration routines
  *
- * Copyright (C) 1999-2004  David I. Bell and Landon Curt Noll
+ * Copyright (C) 1999-2006  David I. Bell and Landon Curt Noll
  *
  * Primary author:  David I. Bell
  *
@@ -19,8 +19,8 @@
  * received a copy with calc; if not, write to Free Software Foundation, Inc.
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
  *
- * @(#) $Revision: 29.15 $
- * @(#) $Id: config.c,v 29.15 2005/12/12 06:45:41 chongo Exp $
+ * @(#) $Revision: 29.22 $
+ * @(#) $Id: config.c,v 29.22 2006/06/25 22:05:38 chongo Exp $
  * @(#) $Source: /usr/local/src/cmd/calc/RCS/config.c,v $
  *
  * Under source code control:	1991/07/20 00:21:56
@@ -97,6 +97,9 @@ NAMETYPE configs[] = {
 	{"compile_custom",	CONFIG_COMPILE_CUSTOM},
 	{"allow_custom",	CONFIG_ALLOW_CUSTOM},
 	{"version",	CONFIG_VERSION},
+	{"baseb",	CONFIG_BASEB},
+	{"redecl_warn",	CONFIG_REDECL_WARN},
+	{"dupvar_warn",	CONFIG_DUPVAR_WARN},
 	{NULL,		0}
 };
 
@@ -159,7 +162,10 @@ CONFIG oldstd = {	/* backward compatible standard configuration */
 	FALSE,			/* compiled without -DCUSTOM */
 #endif
 	&allow_custom,		/* *TRUE=> custom functions are enabled */
-	NULL			/* version */
+	NULL,			/* version */
+	BASEB,			/* base for calculations */
+	TRUE,			/* warn when redeclaring */
+	TRUE,			/* warn when variable names collide */
 };
 CONFIG newstd = {	/* new non-backward compatible configuration */
 	MODE_INITIAL,		/* current output mode */
@@ -216,7 +222,10 @@ CONFIG newstd = {	/* new non-backward compatible configuration */
 	FALSE,			/* compiled without -DCUSTOM */
 #endif
 	&allow_custom,		/* *TRUE=> custom functions are enabled */
-	NULL			/* version */
+	NULL,			/* version */
+	BASEB,			/* base for calculations */
+	TRUE,			/* warn when redeclaring */
+	TRUE,			/* warn when variable names collide */
 };
 CONFIG *conf = NULL;	/* loaded in at startup - current configuration */
 
@@ -405,7 +414,8 @@ lookup_name(NAMETYPE *set, long val)
 /*
  * Check whether VALUE at vp is a LEN (32-bit signed integer) and if so,
  * copy that integer to lp.
- * Return: 1, 2, 0, or -1 XXX
+ *
+ * Return: 1 ==> not an integer, 2 ==> int > 2^31, 0 ==> OK, -1 ==> error
  */
 
 static int
@@ -527,7 +537,7 @@ setconfig(int type, VALUE *vp)
 		break;
 
 	case CONFIG_MUL2:
-		if (getlen(vp, &len)) {
+		if (getlen(vp, &len) || len < 0 || len == 1) {
 			math_error("Bad value for mul2");
 			/*NOTREACHED*/
 		}
@@ -537,7 +547,7 @@ setconfig(int type, VALUE *vp)
 		break;
 
 	case CONFIG_SQ2:
-		if (getlen(vp, &len)) {
+		if (getlen(vp, &len) || len < 0 || len == 1) {
 			math_error("Bad value for sq2");
 			/*NOTREACHED*/
 		}
@@ -547,7 +557,7 @@ setconfig(int type, VALUE *vp)
 		break;
 
 	case CONFIG_POW2:
-		if (getlen(vp, &len)) {
+		if (getlen(vp, &len) || len < 0 || len == 1) {
 			math_error("Bad value for pow2");
 			/*NOTREACHED*/
 		}
@@ -557,7 +567,7 @@ setconfig(int type, VALUE *vp)
 		break;
 
 	case CONFIG_REDC2:
-		if (getlen(vp, &len)) {
+		if (getlen(vp, &len) || len < 0 || len == 1) {
 			math_error("Bad value for redc2");
 			/*NOTREACHED*/
 		}
@@ -722,7 +732,7 @@ setconfig(int type, VALUE *vp)
 			math_error("Cannot duplicate new prompt");
 			/*NOTREACHED*/
 		}
-		strcpy(p, vp->v_str->s_str);
+		strncpy(p, vp->v_str->s_str, vp->v_str->s_len + 1);
 		free(conf->prompt1);
 		conf->prompt1 = p;
 		break;
@@ -737,7 +747,7 @@ setconfig(int type, VALUE *vp)
 			math_error("Cannot duplicate new more prompt");
 			/*NOTREACHED*/
 		}
-		strcpy(p, vp->v_str->s_str);
+		strncpy(p, vp->v_str->s_str, vp->v_str->s_len + 1);
 		free(conf->prompt2);
 		conf->prompt2 = p;
 		break;
@@ -899,6 +909,38 @@ setconfig(int type, VALUE *vp)
 		math_error("The version config parameter is read-only");
 		/*NOTREACHED*/
 
+	case CONFIG_BASEB:
+		math_error("The baseb config parameter is read-only");
+		/*NOTREACHED*/
+
+	case CONFIG_REDECL_WARN:
+		if (vp->v_type == V_NUM) {
+			q = vp->v_num;
+			conf->redecl_warn = !qiszero(q);
+		} else if (vp->v_type == V_STR) {
+			temp = lookup_long(truth, vp->v_str->s_str);
+			if (temp < 0) {
+				math_error("Illegal truth value for redecl_warn");
+				/*NOTREACHED*/
+			}
+			conf->redecl_warn = (int)temp;
+		}
+		break;
+
+	case CONFIG_DUPVAR_WARN:
+		if (vp->v_type == V_NUM) {
+			q = vp->v_num;
+			conf->dupvar_warn = !qiszero(q);
+		} else if (vp->v_type == V_STR) {
+			temp = lookup_long(truth, vp->v_str->s_str);
+			if (temp < 0) {
+				math_error("Illegal truth value for dupvar_warn");
+				/*NOTREACHED*/
+			}
+			conf->dupvar_warn = (int)temp;
+		}
+		break;
+
 	default:
 		math_error("Setting illegal config parameter");
 		/*NOTREACHED*/
@@ -1110,20 +1152,12 @@ config_value(CONFIG *cfg, int type, VALUE *vp)
 		break;
 
 	case CONFIG_TILDE:
-		if (cfg->tilde_ok) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->tilde_ok ? 1 : 0);
+		break;
 
 	case CONFIG_TAB:
-		if (cfg->tab_ok) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->tab_ok ? 1 : 0);
+		break;
 
 	case CONFIG_QUOMOD:
 		i = cfg->quomod;
@@ -1162,20 +1196,12 @@ config_value(CONFIG *cfg, int type, VALUE *vp)
 		break;
 
 	case CONFIG_LEADZERO:
-		if (cfg->leadzero) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->leadzero ? 1 : 0);
+		break;
 
 	case CONFIG_FULLZERO:
-		if (cfg->fullzero) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->fullzero ? 1 : 0);
+		break;
 
 	case CONFIG_MAXSCAN:
 		i = cfg->maxscancount;
@@ -1196,12 +1222,8 @@ config_value(CONFIG *cfg, int type, VALUE *vp)
 		break;
 
 	case CONFIG_BLKVERBOSE:
-		if (cfg->blkverbose) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->blkverbose ? 1 : 0);
+		break;
 
 	case CONFIG_BLKBASE:
 		vp->v_type = V_STR;
@@ -1236,12 +1258,8 @@ config_value(CONFIG *cfg, int type, VALUE *vp)
 		break;
 
 	case CONFIG_VERBOSE_QUIT:
-		if (cfg->verbose_quit) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->verbose_quit ? 1 : 0);
+		break;
 
 	case CONFIG_CTRL_D:
 		vp->v_type = V_STR;
@@ -1272,40 +1290,24 @@ config_value(CONFIG *cfg, int type, VALUE *vp)
 		return;
 
 	case CONFIG_WINDOWS:
-		if (cfg->windows) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->windows ? 1 : 0);
+		break;
 
 	case CONFIG_CYGWIN:
-		if (cfg->cygwin) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->cygwin ? 1 : 0);
+		break;
 
 	case CONFIG_COMPILE_CUSTOM:
-		if (cfg->compile_custom) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (cfg->compile_custom ? 1 : 0);
+		break;
 
 	case CONFIG_ALLOW_CUSTOM:
 		/* firewall */
 		if (cfg->allow_custom == NULL) {
 			cfg->allow_custom = &allow_custom;
 		}
-		if (*(cfg->allow_custom)) {
-			vp->v_num = itoq(1);
-		} else {
-			vp->v_num = itoq(0);
-		}
-		return;
+		i = (*(cfg->allow_custom) ? 1 : 0);
+		break;
 
 	case CONFIG_VERSION:
 		vp->v_type = V_STR;
@@ -1316,6 +1318,18 @@ config_value(CONFIG *cfg, int type, VALUE *vp)
 		}
 		return;
 
+	case CONFIG_BASEB:
+		i = BASEB;
+		break;
+
+	case CONFIG_REDECL_WARN:
+		i = (cfg->redecl_warn ? 1 : 0);
+		break;
+
+	case CONFIG_DUPVAR_WARN:
+		i = (cfg->dupvar_warn ? 1 : 0);
+		break;
+
 	default:
 		math_error("Getting illegal CONFIG element");
 		/*NOTREACHED*/
@@ -1420,5 +1434,7 @@ config_cmp(CONFIG *cfg1, CONFIG *cfg2)
 	       (cfg1->version == NULL && cfg2->version != NULL) ||
 	       (cfg1->version != NULL && cfg2->version == NULL) ||
 	       (cfg1->version != NULL && cfg2->version != NULL &&
-		strcmp(cfg1->version, cfg2->version) != 0);
+		strcmp(cfg1->version, cfg2->version) != 0) ||
+
+	       cfg1->baseb != cfg2->baseb;
 }
